/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_nvlipt_ip_h__
#define __ls10_dev_nvlipt_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_NVLIPT_COMMON          0x000007FF:0x00000000 /* RW--D */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_CHIP_SID_LO           0x00000108      /* R--4R */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_CHIP_SID_LO_SID_31_0  31:0            /* R-IVF */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_CHIP_SID_LO_SID_31_0_INIT 0x00000000  /* R-I-V */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_CHIP_SID_HI           0x0000010c      /* R--4R */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_CHIP_SID_HI_SID_63_32 31:0            /* R-IVF */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_CHIP_SID_HI_SID_63_32_INIT 0x00000000 /* R-I-V */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION    0x00000114      /* RW-4R */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION_NUM_LINKS_PER_NVLIPT 7:0 /* R-IVF */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION_NUM_LINKS_PER_NVLIPT_INIT 0x00000004 /* R-I-V */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION_NUM_LANES_PER_LINK 15:8 /* R-IVF */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION_NUM_LANES_PER_LINK_INIT 0x00000002 /* R-I-V */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION_NUM_USABLE_LINKS 26:24 /* RWIVF */
#define NV_NVLIPT_COMMON_TOPOLOGY_LOCAL_LINK_CONFIGURATION_NUM_USABLE_LINKS_INIT 0x00000004 /* RWI-V */
#define NV_NVLIPT_COMMON_ERR_STATUS_0                         0x00000280      /* RW-4R */
#define NV_NVLIPT_COMMON_ERR_STATUS_0_CLKCTL_ILLEGAL_REQUEST  0:0             /* RWIVF */
#define NV_NVLIPT_COMMON_ERR_STATUS_0_CLKCTL_ILLEGAL_REQUEST__ONWRITE "oneToClear" /*       */
#define NV_NVLIPT_COMMON_ERR_STATUS_0_CLKCTL_ILLEGAL_REQUEST_NONE 0x00000000  /* RWI-V */
#define NV_NVLIPT_COMMON_ERR_STATUS_0_CLKCTL_ILLEGAL_REQUEST_CLEAR 0x00000001 /* RW--V */
#define NV_NVLIPT_COMMON_ERR_FATAL_REPORT_EN_0                0x00000288      /* RW-4R */
#define NV_NVLIPT_COMMON_ERR_FATAL_REPORT_EN_0_CLKCTL_ILLEGAL_REQUEST 0:0     /* RWEVF */
#define NV_NVLIPT_COMMON_ERR_FATAL_REPORT_EN_0_CLKCTL_ILLEGAL_REQUEST_DISABLE 0x00000000 /* RWE-V */
#define NV_NVLIPT_COMMON_ERR_FATAL_REPORT_EN_0_CLKCTL_ILLEGAL_REQUEST_ENABLE 0x00000001 /* RW--V */
#define NV_NVLIPT_COMMON_ERR_CONTAIN_EN_0                     0x00000294      /* RW-4R */
#define NV_NVLIPT_COMMON_ERR_CONTAIN_EN_0_CLKCTL_ILLEGAL_REQUEST 0:0          /* RWEVF */
#define NV_NVLIPT_COMMON_ERR_CONTAIN_EN_0_CLKCTL_ILLEGAL_REQUEST_DISABLE 0x00000000 /* RWE-V */
#define NV_NVLIPT_COMMON_ERR_CONTAIN_EN_0_CLKCTL_ILLEGAL_REQUEST_ENABLE 0x00000001 /* RW--V */
#define NV_NVLIPT_COMMON_ERR_FIRST_0                          0x0000029c      /* RW-4R */
#define NV_NVLIPT_COMMON_ERR_FIRST_0_CLKCTL_ILLEGAL_REQUEST   0:0             /* RWIVF */
#define NV_NVLIPT_COMMON_ERR_FIRST_0_CLKCTL_ILLEGAL_REQUEST__ONWRITE "oneToClear" /*       */
#define NV_NVLIPT_COMMON_ERR_FIRST_0_CLKCTL_ILLEGAL_REQUEST_NONE 0x00000000   /* RWI-V */
#define NV_NVLIPT_COMMON_ERR_FIRST_0_CLKCTL_ILLEGAL_REQUEST_CLEAR 0x00000001  /* RW--V */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON                  0x00000300      /* RW-4R */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON_INT0_EN          0:0             /* RWEVF */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON_INT0_EN_DISABLE  0x00000000      /* RWE-V */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON_INT0_EN_ENABLE   0x00000001      /* RW--V */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON_INT1_EN          1:1             /* RWEVF */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON_INT1_EN_DISABLE  0x00000000      /* RWE-V */
#define NV_NVLIPT_COMMON_INTR_CONTROL_COMMON_INT1_EN_ENABLE   0x00000001      /* RW--V */
#endif // __ls10_dev_nvlipt_ip_h__
